library(dplyr)
library(ggplot2)
library(reshape2)
library(ggpubr)
library(tidybayes)
library(HDInterval)
# Contact: Philip Gould, pgould@usgs.gov
# Overview
## The below code executes the power analysis associated with: "Coastal Sage Scrub and Chaparral Community Monitoring Plan for Western San Diego County"
## The datsum object contains the average cover values for 4 cover classes from data collected from 1995 - 2020 in SD county, CA, USA.
## The power analysis estimates the required sample size to detect significant changes in cover for Non-native Grass ("NNG") and Shrub ("Shrub") cover.
## 5 positive and negative change scenarios are included to evaluate the relationship between absolute cover change, sample size, and power.
# Monte-Carlo Simulation #
sims <- 100
sim_power <- data.frame(X1 = rep(c(rep("CSS",75),
                                   rep("Chaparral",75)),2),
                        year = c(rep(1,150),
                                 rep(2,150)))
# To describe uncertainty present in the simulation, we can generate simulated data sets. We then run the anova analysis for each simulated data set with confidence intervals. We chose to display 95% intervals, i.e. the range of values present in 95% of the analyses.
hist_dat <- read.csv("hist_dat.csv")
datsum <- read.csv("data_summary.csv")

# View the distribution of the historical data that is loaded in. 1997 data from RANGELAND dataset
# Annual herbaceous (i.e. annual non-native grass) distribution for rangeland data from 1997.
hist_dat[,c(2,7:10)] %>% ggplot(aes(x = Y1997_annH)) + geom_histogram(alpha = 0.5, color = "black")
# Annual herbaceous (i.e. annual non-native grass) distribution for rangeland data from 1997.
hist_dat[,c(2,7:10)] %>% ggplot(aes(x = Y1997_shru)) + geom_histogram(alpha = 0.5, color = "black")

# Process data for simulations
datsum <- datsum[,-1]
set.seed(2011)

# Define multipliers for augmentation; i.e. Start with "year" 1 data and augment +/- given the defined multipliers
# These are our estimates of "true" change that may or may not be detected in the below simulations.
# We estimate power to detect nonnative grass positive change (aug) and negative change (aug2) scenarios.
# We estimate power to detect native shrub positive change (aug4) and negative change (aug3) scenarios.
aug <- c(1.2,1.4,1.6,1.8,2.0)
aug2 <- c(0.9,0.8,0.7,0.6,0.5)
aug3 <- c(0.9,0.8,0.7,0.6,0.5)
aug4 <- c(1.05,1.10,1.15,1.20,1.25)
# create empty data storage objects to populate in for loops below.
# each row = site
# each col = cover type: 
## col1 = CSS/Chaparral ID; 
## col2 = Non-native Grass
## col3 = Native Shrub
## col4 = Open Ground (Not used hear but simulated)
# The structure is 150 rows, by 4 cover type cols, by 5 aug scenarios, by sims
# the 150 rows corresponds to 1:75 = CSS sites and 76:150 = Chaparral sites.
mc <- array(data = NA,dim = c(150,4,5,sims)) # stores initial starting data
mc2 <- array(data = NA,dim = c(150,4,5,sims)) # stores "year" 2 data when change = aug/2
mc3 <- array(data = NA,dim = c(150,4,5,sims)) # stores "year" 2 data when change = aug3/4
# simulate data
for(a in 1:5){ # %change scenarios defined by aug objects
  for(i in 1:sims) {
    for (j in 1:75) {
      mc[j, 1, a, i] <- 1
      mc[j+75, 1, a, i] <- 2
      mc[j, 2, a, i] <- rnorm(75,mean = data.frame(datsum)[1,2],sd = data.frame(datsum)[1,3])[j]
      mc[j+75, 2, a, i] <- rnorm(75,mean = data.frame(datsum)[2,2],sd = data.frame(datsum)[2,3])[j]
      mc[j, 3, a, i] <- rnorm(75,mean = data.frame(datsum)[1,4],sd = data.frame(datsum)[1,5])[j]
      mc[j+75, 3, a, i] <- rnorm(75,mean = data.frame(datsum)[2,4],sd = data.frame(datsum)[2,5])[j]
      mc[j, 4, a, i] <- rnorm(75,mean = data.frame(datsum)[1,6],sd = data.frame(datsum)[1,7])[j]
      mc[j+75, 4, a, i] <- rnorm(75,mean = data.frame(datsum)[2,6],sd = data.frame(datsum)[2,7])[j]
    }
  }
}
# force data to be censored at 0, as we cannot observe -% cover data. This results in 0 truncated normal data.
mc <- ifelse(mc<0,0,mc)

# Simulate data for "year" 2, i.e. post-augmentation data.
# here we take the previous year's data for each site and
# a normally distributed random change with mean = aug/2/3/4
for(a in 1:5){ # %change scenarios defined by aug objects
  for(i in 1:sims) {
    for (j in 1:75) {
      mc2[j, 1, a, i] <- 1
      mc2[j+75, 1, a, i] <- 2
      mc2[j, 2, a, i] <- mc[j,2,a,i]*rnorm(1,mean = aug[a],sd = 0.25)
      mc2[j+75, 2, a, i] <- mc[j+75,2,a,i]*rnorm(1,mean = aug[a],sd = 0.25)
      mc2[j, 3, a, i] <- mc[j,3,a,i]*rnorm(1,mean = aug2[a],sd = 0.25)
      mc2[j+75, 3, a, i] <- mc[j+75,3,a,i]*rnorm(1,mean = aug2[a],sd = 0.25)
      mc2[j, 4, a, i] <- mc[j,4,a,i]*rnorm(1,mean = aug2[a],sd = 0.25)
      mc2[j+75, 4, a, i] <- mc[j+75,4,a,i]*rnorm(1,mean = aug2[a],sd = 0.25)

      
      mc3[j, 1, a, i] <- 1
      mc3[j+75, 1, a, i] <- 2
      mc3[j, 2, a, i] <- mc[j,2,a,i]*rnorm(1,mean = aug3[a],sd = 0.25)
      mc3[j+75, 2, a, i] <- mc[j+75,2,a,i]*rnorm(1,mean = aug3[a],sd = 0.25)
      mc3[j, 3, a, i] <- mc[j,3,a,i]*rnorm(1,mean = aug4[a],sd = 0.25)
      mc3[j+75, 3, a, i] <- mc[j+75,3,a,i]*rnorm(1,mean = aug4[a],sd = 0.25)
      mc3[j, 4, a, i] <- mc[j,4,a,i]*rnorm(1,mean = aug4[a],sd = 0.25)
      mc3[j+75, 4, a, i] <- mc[j+75,4,a,i]*rnorm(1,mean = aug4[a],sd = 0.25)
    }
  }
}
# bind arrays together along the row dim
# mc_power = data from aug = aug/aug2
mc_power <- abind::abind(mc,mc2,along = 1)
# ensure all data >= 0
mc_power <- ifelse(mc_power<0,0,mc_power)
# mc_power = data from aug = aug3/aug4
mc_power2 <- abind::abind(mc,mc3,along = 1)
mc_power2 <- ifelse(mc_power2<0,0,mc_power2)
# store data for NNG and shrub in different objects
a_mc <- array(NA,dim = c(75,3,5,100))
a_mc_shru <- array(NA,dim = c(75,3,5,100))
colnames(a_mc) <- c("sample","CSS","Chaparral")
for(a in 1:5){
  for(i in 1:sims){
    for(n in 2:75){
      a_mc[n, 2, a, i] <- summary(aov(mc_power[sim_power$X1 == "CSS", 2, a, i][c(1:n,(75 + 1):(75 + n))] ~ factor(sim_power$year[sim_power$X1 == "CSS"])[c(1:n,(75 + 1):(75 + n))]))[[1]][1,5]
      a_mc[n, 3, a, i] <- summary(aov(mc_power[sim_power$X1 == "Chaparral", 2, a, i][c(1:n,(75 + 1):(75 + n))] ~ factor(sim_power$year[sim_power$X1 == "Chaparral"])[c(1:n,(75 + 1):(75 + n))]))[[1]][1,5]
      a_mc_shru[n, 2, a, i] <- summary(aov(mc_power[sim_power$X1 == "CSS", 3, a, i][c(1:n,(75 + 1):(75 + n))] ~ factor(sim_power$year[sim_power$X1 == "CSS"])[c(1:n,(75 + 1):(75 + n))]))[[1]][1,5]
      a_mc_shru[n, 3, a, i] <- summary(aov(mc_power[sim_power$X1 == "Chaparral", 3, a, i][c(1:n,(75 + 1):(75 + n))] ~ factor(sim_power$year[sim_power$X1 == "Chaparral"])[c(1:n,(75 + 1):(75 + n))]))[[1]][1,5]
    }
  }
}
a_mc[,1,,] <- 1:75
# To save if time consuming: saveRDS(a_mc,"a_mc2.RDS")
# store data for NNG and shrub in different objects; aug3/4 scenarios
a_mc3 <- array(NA,dim = c(75,3,5,100))
a_mc3_shru <- array(NA,dim = c(75,3,5,100))
colnames(a_mc) <- c("sample","CSS","Chaparral")
for(a in 1:5){
  for(i in 1:sims){
    for(n in 2:75){
      a_mc3[n, 2, a, i] <- summary(aov(mc_power2[sim_power$X1 == "CSS", 2, a, i][c(1:n,(75 + 1):(75 + n))] ~ factor(sim_power$year[sim_power$X1 == "CSS"])[c(1:n,(75 + 1):(75 + n))]))[[1]][1,5]
      a_mc3[n, 3, a, i] <- summary(aov(mc_power2[sim_power$X1 == "Chaparral", 2, a, i][c(1:n,(75 + 1):(75 + n))] ~ factor(sim_power$year[sim_power$X1 == "Chaparral"])[c(1:n,(75 + 1):(75 + n))]))[[1]][1,5]
      a_mc3_shru[n, 2, a, i] <- summary(aov(mc_power2[sim_power$X1 == "CSS", 3, a, i][c(1:n,(75 + 1):(75 + n))] ~ factor(sim_power$year[sim_power$X1 == "CSS"])[c(1:n,(75 + 1):(75 + n))]))[[1]][1,5]
      a_mc3_shru[n, 3, a, i] <- summary(aov(mc_power2[sim_power$X1 == "Chaparral", 3, a, i][c(1:n,(75 + 1):(75 + n))] ~ factor(sim_power$year[sim_power$X1 == "Chaparral"])[c(1:n,(75 + 1):(75 + n))]))[[1]][1,5]
    }
  }
}
a_mc3[,1,,] <- 1:75
colnames(a_mc3) <- c("sample","CSS","Chaparral")
# To save if time consuming: saveRDS(a_mc3,"a_mc3.RDS")


# To read if time consuming: a_mc <- readRDS("a_mc2.RDS")
# To read if time consuming: a_mc3 <- readRDS("a_mc3.RDS")


# Next, we review power, as defined by @irvine2011. In this context, Power is defined as the proportion of simulations where $\alpha$ \< 0.05. The distinction here is that this is a summary of the previous estimate, which showed the distribution of $\alpha$ for each sample size. Here instead, we consider the conclusion we draw from observing $\alpha$ \< 0.05. Subsequently, the value can be interpreted as the average likelihood of estimating correctly that vegetative community composition differed between years. This is more similar to traditional representations of power.

# Figures for NNG and Shrub
ggarrange(legend = "right",
          common.legend = T,
          ncol = 1,
          melt(a_mc[,2:3,,]) %>% 
            group_by(Var1,Var2,Var3) %>% 
            summarise(value = sum(value<0.05,na.rm=T)/sims) %>% 
            ggplot(aes(x = Var1, y = value)) + 
            geom_point(alpha = 0.5) + 
            geom_smooth(aes(linetype = factor(Var2),
                            color = factor(Var3, labels = c("+10% NNG","+20% NNG","+30% NNG","+40% NNG","+50% NNG"))),se=F) + 
            geom_hline(yintercept = 0.8, linetype = "dashed") + 
            ylab("Power") + xlab("Sample Size") + 
            theme(legend.title = element_blank(), legend.spacing.y = unit(.3,'cm'), 
                  strip.text = element_text(size = 14), 
                  plot.title = element_text(size = 20), 
                  axis.title = element_text(size = 14), 
                  axis.text = element_text(size = 12), 
                  legend.text = element_text(size = 12)) +
            facet_grid(~factor(Var3, labels = c("+10% NNG","+20% NNG","+30% NNG","+40% NNG","+50% NNG"))) + 
            guides(color = element_blank()) + 
            ggtitle("Increase in Grass Cover") + scale_color_discrete(guide = "none"),
          melt(a_mc3[,2:3,,]) %>% 
            group_by(Var1,Var2,Var3) %>% 
            summarise(value = sum(value<0.05,na.rm=T)/sims) %>% 
            ggplot(aes(x = Var1, y = value)) + 
            geom_point(alpha = 0.5) + 
            geom_smooth(aes(linetype = factor(Var2),
                            color = factor(Var3, labels = c("-10% NNG","-20% NNG","-30% NNG","-40% NNG","-50% NNG"))),se=F) + 
            geom_hline(yintercept = 0.8, linetype = "dashed") + 
            ylab("Power") + xlab("Sample Size") + 
            theme(legend.title = element_blank(), legend.spacing.y = unit(.3,'cm'), 
                  strip.text = element_text(size = 14), 
                  plot.title = element_text(size = 20), 
                  axis.title = element_text(size = 14), 
                  axis.text = element_text(size = 12), 
                  legend.text = element_text(size = 12)) + 
            facet_grid(~factor(Var3, labels = c("-10% NNG","-20% NNG","-30% NNG","-40% NNG","-50% NNG"))) + 
            guides(color = element_blank()) + 
            ggtitle("Decrease in Grass Cover") + 
            scale_color_discrete(guide = "none")
)
#ggsave("powerAnalysis.png",device = "png",dpi = 300,bg = "white")

ggarrange(legend = "right",
          common.legend = T,
          ncol = 1,
          melt(a_mc3_shru[,2:3,,]) %>% 
            group_by(Var1,Var2,Var3) %>% 
            summarise(value = sum(value<0.05,na.rm=T)/sims) %>% 
            ggplot(aes(x = Var1, y = value)) + 
            geom_point(alpha = 0.5) + 
            geom_smooth(aes(linetype = factor(Var2),
                            color = factor(Var3, labels = c("+5% Shrub","+10% Shrub","+15% Shrub","+20% Shrub","+25% Shrub"))),se=F) + 
            geom_hline(yintercept = 0.8, linetype = "dashed") + 
            ylab("Power") + xlab("Sample Size") + 
            theme(legend.title = element_blank(), legend.spacing.y = unit(.3,'cm'), 
                  strip.text = element_text(size = 14), 
                  plot.title = element_text(size = 20), 
                  axis.title = element_text(size = 14), 
                  axis.text = element_text(size = 12), 
                  legend.text = element_text(size = 12)) +
            facet_grid(~factor(Var3, labels = c("+5% Shrub","+10% Shrub","+15% Shrub","+20% Shrub","+25% Shrub"))) + 
            guides(color = element_blank()) + 
            ggtitle("Increase in Native Shrub Cover") + 
            scale_color_discrete(guide = "none"),
          melt(a_mc_shru[,2:3,,]) %>% 
            group_by(Var1,Var2,Var3) %>% 
            summarise(value = sum(value<0.05,na.rm=T)/sims) %>% 
            ggplot(aes(x = Var1, y = value)) + 
            geom_point(alpha = 0.5) + 
            geom_smooth(aes(linetype = factor(Var2),
                            color = factor(Var3, labels = c("-10% Shrub","-20% Shrub","-30% Shrub","-40% Shrub","-50% Shrub"))),se=F) +
            geom_hline(yintercept = 0.8, linetype = "dashed") + 
            ylab("Power") + xlab("Sample Size") + 
            theme(legend.title = element_blank(), legend.spacing.y = unit(.3,'cm'), 
                    strip.text = element_text(size = 14), 
                    plot.title = element_text(size = 20), 
                    axis.title = element_text(size = 14), 
                    axis.text = element_text(size = 12), 
                    legend.text = element_text(size = 12)) +
            facet_grid(~factor(Var3, labels = c("-10% Shrub","-20% Shrub","-30% Shrub","-40% Shrub","-50% Shrub"))) + 
            guides(color = element_blank()) + 
            ggtitle("Decrease in Native Shrub cover") + 
            scale_color_discrete(guide = "none")
)
#ggsave("powerAnalysis_Shrub.png",device = "png",dpi = 300,bg = "white")


### estimate site ratio CSS:Chap
#negative change
melt(a_mc[, 2:3, , ]) %>%
  group_by(Var2, Var1, Var3) %>%
  summarise(
    mean = mean((value) < 0.05, na.rm = T),
    LCI = HDInterval::hdi(value, na.rm = T)[1],
    UCI = HDInterval::hdi(value, na.rm = T)[2]
  ) %>%
  ungroup() %>%
  group_by(Var2, Var3) %>%
  filter(mean >= 0.80) %>%
  slice(1)
#positive change
melt(a_mc3[, 2:3, , ]) %>% group_by(Var2, Var1, Var3) %>% summarise(
  mean = mean((value) < 0.05, na.rm = T),
  LCI = HDInterval::hdi(value, na.rm = T)[1],
  UCI = HDInterval::hdi(value, na.rm = T)[2]
) %>% ungroup() %>% group_by(Var2, Var3) %>% filter(mean >= 0.80) %>% slice(1)



#Chaparral power given 45 sites
melt(a_mc[, 2:3, , ]) %>% group_by(Var2, Var1, Var3) %>% summarise(
  mean = mean((value) < 0.05, na.rm = T),
  LCI = HDInterval::hdi(value, na.rm = T)[1],
  UCI = HDInterval::hdi(value, na.rm = T)[2]
) %>% ungroup() %>% group_by(Var2, Var3) %>% filter(Var1 == 45)
#CSS power given 55 sites
melt(a_mc[, 2:3, , ]) %>% group_by(Var2, Var1, Var3) %>% summarise(
  mean = mean((value) < 0.05, na.rm = T),
  LCI = HDInterval::hdi(value, na.rm = T)[1],
  UCI = HDInterval::hdi(value, na.rm = T)[2]
) %>% ungroup() %>% group_by(Var2, Var3) %>% filter(Var1 == 55)


#Chaparral power given 45 sites
melt(a_mc3[, 2:3, , ]) %>% group_by(Var2, Var1, Var3) %>% summarise(
  mean = mean((value) < 0.05, na.rm = T),
  LCI = HDInterval::hdi(value, na.rm = T)[1],
  UCI = HDInterval::hdi(value, na.rm = T)[2]
) %>% ungroup() %>% group_by(Var2, Var3) %>% filter(Var1 == 45)
#CSS power given 55 sites
melt(a_mc3[, 2:3, , ]) %>% group_by(Var2, Var1, Var3) %>% summarise(
  mean = mean((value) < 0.05, na.rm = T),
  LCI = HDInterval::hdi(value, na.rm = T)[1],
  UCI = HDInterval::hdi(value, na.rm = T)[2]
) %>% ungroup() %>% group_by(Var2, Var3) %>% filter(Var1 == 55)

